@@ -62,7 +62,7 @@ class ProductPassport:
 
         tx = Contract.constructor(initial_owner or self.account.address).build_transaction({
             'from': self.account.address,
-            'nonce': self.web3.eth.get_transaction_count(self.account.address),
+            'nonce': self.web3.eth.get_transaction_count(self.account.address, 'pending'),
             'gas': estimated_gas,
             'gasPrice': self.web3.to_wei(self.gwei_bid, 'gwei')
         })
@@ -77,6 +77,40 @@ class ProductPassport:
         self.logger.info(f"ProductPassport contract deployed at address: {contract_address}")
         return contract_address
 
+    def authorize_entity(self, contract_address, entity_address):
+        """
+        Authorizes an entity to interact with the ProductPassport contract.
+
+        Args:
+            contract_address (str): The address of the ProductPassport contract.
+            entity_address (str): The address of the entity to authorize.
+
+        Returns:
+            dict: The transaction receipt containing details of the transaction.
+        
+        Raises:
+            ValueError: If the transaction fails.
+        """
+        contract = self.web3.eth.contract(address=contract_address, abi=self.contract['abi'])
+        
+        try:
+            tx = contract.functions.authorizeEntity(entity_address).build_transaction({
+                'from': self.account.address,
+                'nonce': self.web3.eth.get_transaction_count(self.account.address, 'pending'),
+                'gas': contract.functions.authorizeEntity(entity_address).estimate_gas({'from': self.account.address}),
+                'gasPrice': self.web3.to_wei(self.gwei_bid, 'gwei')
+            })
+            
+            signed_tx = self.web3.eth.account.sign_transaction(tx, self.account.key)
+            tx_hash = self.web3.eth.send_raw_transaction(signed_tx.raw_transaction)
+            tx_receipt = self.web3.eth.wait_for_transaction_receipt(tx_hash)
+
+            self.logger.info(f"Entity authorized transaction receipt: {tx_receipt}")
+            return tx_receipt
+        except Exception as e:
+            self.logger.error(f"Failed to authorize entity: {e}")
+            raise
+
     def set_product(self, contract_address, product_id, product_details):
         """
         Sets the product details in the ProductPassport contract.
@@ -96,7 +130,7 @@ class ProductPassport:
         contract = self.web3.eth.contract(address=contract_address, abi=self.product_details_contract['abi'])
 
         try:
-            tx_hash = contract.functions.setProduct(
+            tx = contract.functions.setProduct(
                 product_id,
                 product_details["uid"],
                 product_details["gtin"],
@@ -104,14 +138,27 @@ class ProductPassport:
                 product_details["manufacturerInfo"],
                 product_details["consumerInfo"],
                 product_details["endOfLifeInfo"]
-            ).transact({
-                'from': self.account.address
+            ).build_transaction({
+                'from': self.account.address,
+                'nonce': self.web3.eth.get_transaction_count(self.account.address, 'pending'),
+                'gas': contract.functions.setProduct(
+                    product_id,
+                    product_details["uid"],
+                    product_details["gtin"],
+                    product_details["taricCode"],
+                    product_details["manufacturerInfo"],
+                    product_details["consumerInfo"],
+                    product_details["endOfLifeInfo"]
+                ).estimate_gas({'from': self.account.address}),
+                'gasPrice': self.web3.to_wei(self.gwei_bid, 'gwei')
             })
-
+            
+            signed_tx = self.web3.eth.account.sign_transaction(tx, self.account.key)
+            tx_hash = self.web3.eth.send_raw_transaction(signed_tx.raw_transaction)
             tx_receipt = self.web3.eth.wait_for_transaction_receipt(tx_hash)
+
             self.logger.info(f"Product set transaction receipt: {tx_receipt}")
             return tx_receipt
-
         except Exception as e:
             self.logger.error(f"Failed to set product: {e}")
             raise
@@ -132,7 +179,7 @@ class ProductPassport:
         """
         contract = self.web3.eth.contract(address=contract_address, abi=self.product_details_contract['abi'])
         try:
-            product = contract.functions.getProduct(product_id)().call()
+            product = contract.functions.getProduct(product_id).call()
             self.logger.info(f"Product retrieved: {product}")
             return product
         except Exception as e:
@@ -159,7 +206,7 @@ class ProductPassport:
         contract = self.web3.eth.contract(address=contract_address, abi=self.contract['abi'])
 
         try:
-            tx_hash = contract.functions.setProductData(
+            tx = contract.functions.setProductData(
                 int(product_id),
                 product_data["description"],
                 product_data["manuals"],
@@ -171,14 +218,31 @@ class ProductPassport:
                 product_data["warrantyInfo"],
                 product_data["materialComposition"],
                 product_data["complianceInfo"]
-            ).transact({
-                'from': self.account.address
+            ).build_transaction({
+                'from': self.account.address,
+                'nonce': self.web3.eth.get_transaction_count(self.account.address, 'pending'),
+                'gas': contract.functions.setProductData(
+                    int(product_id),
+                    product_data["description"],
+                    product_data["manuals"],
+                    product_data["specifications"],
+                    product_data["batchNumber"],
+                    product_data["productionDate"],
+                    product_data["expiryDate"],
+                    product_data["certifications"],
+                    product_data["warrantyInfo"],
+                    product_data["materialComposition"],
+                    product_data["complianceInfo"]
+                ).estimate_gas({'from': self.account.address}),
+                'gasPrice': self.web3.to_wei(self.gwei_bid, 'gwei')
             })
-
+            
+            signed_tx = self.web3.eth.account.sign_transaction(tx, self.account.key)
+            tx_hash = self.web3.eth.send_raw_transaction(signed_tx.raw_transaction)
             tx_receipt = self.web3.eth.wait_for_transaction_receipt(tx_hash)
+
             self.logger.info(f"Product data set transaction receipt: {tx_receipt}")
             return tx_receipt
-
         except Exception as e:
             self.logger.error(f"Failed to set product data: {e}")
             raise
@@ -205,26 +269,3 @@ class ProductPassport:
         except Exception as e:
             self.logger.error(f"Failed to retrieve product data: {e}")
             raise
-
-    def authorize_entity(self, contract_address, entity_address):
-        """
-        Authorizes an entity to interact with the ProductPassport contract.
-
-        Args:
-            contract_address (str): The address of the ProductPassport contract.
-            entity_address (str): The address of the entity to authorize.
-
-        Returns:
-            dict: The transaction receipt containing details of the transaction.
-        
-        Raises:
-            ValueError: If the transaction fails.
-        """
-        contract = self.web3.eth.contract(address=contract_address, abi=self.contract['abi'])
-        
-        tx_hash = contract.functions.authorizeEntity(entity_address).transact({
-            'from': self.account.address
-        })
-        
-        tx_receipt = self.web3.eth.wait_for_transaction_receipt(tx_hash)
-        return tx_receipt
\ No newline at end of file
